<?php

declare(strict_types=1);

namespace Dedi\SyliusAdvertisingPlugin\Entity;

use Doctrine\ORM\Mapping as ORM;
use Gedmo\Mapping\Annotation as Gedmo;
use Sylius\Component\Core\Model\ImageInterface;
use Sylius\Component\Resource\Model\TranslatableTrait;
use Sylius\Component\Resource\Model\TranslationInterface;
use Symfony\Component\Validator\Constraints as Assert;

/**
 * @ORM\Entity()
 * @ORM\Table(name="dedi_advertising")
 */
class Advertising implements AdvertisingInterface
{
    use TranslatableTrait {
        __construct as private initializeTranslationsCollection;
        getTranslation as private doGetTranslation;
    }

    /**
     * @ORM\Id()
     * @ORM\GeneratedValue(strategy="AUTO")
     * @ORM\Column(type="integer", nullable=false, name="id")
     */
    protected ?int $id = null;

    /**
     * @ORM\Column(type="string", nullable=false, name="advertising_code")
     * @Gedmo\Slug(fields={"id"}, unique=true)
     * @Assert\NotBlank(groups={"default", "sylius"})
     */
    protected ?string $code = null;

    /**
     * @ORM\OneToOne(targetEntity="Dedi\SyliusAdvertisingPlugin\Entity\AdvertisingImage", mappedBy="owner", orphanRemoval=true, cascade={"persist"})
     */
    protected ?ImageInterface $image = null;

    /**
     * @ORM\Column(type="integer", nullable=false, name="advertising_position")
     */
    protected ?int $position = 0;

    /**
     * @ORM\Column(type="boolean", nullable=false, name="advertising_enabled")
     */
    protected bool $enabled = true;

    public function __construct()
    {
        $this->initializeTranslationsCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getCode(): string
    {
        return (string) $this->code;
    }

    public function setCode(?string $code): self
    {
        $this->code = $code;

        return $this;
    }

    public function getPosition(): int
    {
        return $this->position;
    }

    public function setPosition(int $position): self
    {
        $this->position = $position;

        return $this;
    }

    public function isEnabled(): bool
    {
        return $this->enabled;
    }

    public function setEnabled(bool $enabled): self
    {
        $this->enabled = $enabled;

        return $this;
    }

    public function getImage(): ?ImageInterface
    {
        return $this->image;
    }

    public function setImage(?ImageInterface $image): self
    {
        $image->setOwner($this);
        $this->image = $image;

        return $this;
    }

    public function getUrl(): ?string
    {
        return $this->getTranslation()->getUrl();
    }

    public function setUrl(?string $url): self
    {
        $this->getTranslation()->setUrl($url);

        return $this;
    }

    public function getContent(): ?string
    {
        return $this->getTranslation()->getContent();
    }

    public function setContent(?string $content): self
    {
        $this->getTranslation()->setContent($content);

        return $this;
    }

    public function getTitle(): ?string
    {
        return $this->getTranslation()->getTitle();
    }

    public function setTitle(?string $title): self
    {
        $this->getTranslation()->setTitle($title);

        return $this;
    }

    public function getTranslation(?string $locale = null): TranslationInterface
    {
        /** @var AdvertisingTranslation $translation */
        $translation = $this->doGetTranslation($locale);

        return $translation;
    }

    protected function createTranslation(): TranslationInterface
    {
        return new AdvertisingTranslation();
    }

    public function __toString()
    {
        return $this->code;
    }
}
